(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  do funkce OpenLayer doplnn parametr Color (P93)
--------------------------------------------------------------------------------

DODLAT:
1)  interpoltor ikmch pohyb
2)  vcensobn exposice clonky ovlivuje i spojov ry

POZNMKY:
1)

----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY Admap3;

USES DrvTypes, DLL_Util;

TYPE
  ParamIndex_T = (
    PadExpositions);

CONST
  ToolCount = 21;
  Both = [UsageDraw, UsageFlash];
  ToolTable: array [1..ToolCount] of ToolDescr_T = (
    (DescrStr: '(Circle 0.20 mm)'; Shape: CircleTool; Width:  200; Height: 0;  Usage: Both; Code: 46),
    (DescrStr: '(Circle 0.25 mm)'; Shape: CircleTool; Width:  250; Height: 0;  Usage: Both; Code: 42),
    (DescrStr: '(Circle 0.30 mm)'; Shape: CircleTool; Width:  300; Height: 0;  Usage: Both; Code: 40),
    (DescrStr: '(Circle 0.40 mm)'; Shape: CircleTool; Width:  400; Height: 0;  Usage: Both; Code: 35),
    (DescrStr: '(Circle 0.50 mm)'; Shape: CircleTool; Width:  500; Height: 0;  Usage: Both; Code: 30),
    (DescrStr: '(Circle 0.60 mm)'; Shape: CircleTool; Width:  600; Height: 0;  Usage: Both; Code: 28),
    (DescrStr: '(Circle 0.80 mm)'; Shape: CircleTool; Width:  800; Height: 0;  Usage: Both; Code: 24),
    (DescrStr: '(Circle 1.0 mm)';  Shape: CircleTool; Width: 1000; Height: 0;  Usage: Both; Code: 21),
    (DescrStr: '(Circle 1.2 mm)';  Shape: CircleTool; Width: 1200; Height: 0;  Usage: Both; Code: 18),
    (DescrStr: '(Circle 1.3 mm)';  Shape: CircleTool; Width: 1300; Height: 0;  Usage: Both; Code: 17),
    (DescrStr: '(Circle 1.4 mm)';  Shape: CircleTool; Width: 1400; Height: 0;  Usage: Both; Code: 16),
    (DescrStr: '(Circle 1.5 mm)';  Shape: CircleTool; Width: 1500; Height: 0;  Usage: Both; Code: 15),
    (DescrStr: '(Circle 1.6 mm)';  Shape: CircleTool; Width: 1600; Height: 0;  Usage: Both; Code: 13),
    (DescrStr: '(Circle 1.7 mm)';  Shape: CircleTool; Width: 1700; Height: 0;  Usage: Both; Code: 12),
    (DescrStr: '(Circle 1.8 mm)';  Shape: CircleTool; Width: 1800; Height: 0;  Usage: Both; Code: 11),
    (DescrStr: '(Circle 1.9 mm)';  Shape: CircleTool; Width: 1900; Height: 0;  Usage: Both; Code: 10),
    (DescrStr: '(Circle 2.0 mm)';  Shape: CircleTool; Width: 2000; Height: 0;  Usage: Both; Code:  9),
    (DescrStr: '(Circle 2.4 mm)';  Shape: CircleTool; Width: 2400; Height: 0;  Usage: Both; Code:  6),
    (DescrStr: '(Circle 2.8 mm)';  Shape: CircleTool; Width: 2800; Height: 0;  Usage: Both; Code:  2),
    (DescrStr: '(Circle 2.9 mm)';  Shape: CircleTool; Width: 2800; Height: 0;  Usage: Both; Code:  1),
    (DescrStr: '(Circle 3.0 mm)';  Shape: CircleTool; Width: 3000; Height: 0;  Usage: Both; Code:  0));

  DriverDescr: DriverDescr_T = (
    Name:         'Admap 3   PH 1995';
    MajorVersion: 4;
    MinorVersion: 0;
    Capabilities: [Optimizable, DrawLines, FlashPads];
    NoOfParams:   Ord (High (ParamIndex_T)) + 1;
    FilesName:    'Admap3';
    TargetExt:    '';
    NoOfTools:    ToolCount;
    InitialTools: @ ToolTable [1];
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           0;
    Y0:           0;
    X1:           350;
    Y1:           250;
    XDividend:    4;
    YDividend:    4;
    Divisor:      1);

  Params: array [ParamIndex_T] of ParamDescr_T = (
   (DescrStr:  '>Pad Expositions';
    Value:     1;
    ParamType: _Numeric;
    MinVal:    1;
    MaxVal:    10));

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := @ Params [ParamIndex_T (ParamNumber)];
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
  Params [ParamIndex_T (ParamNumber)].Value := ParamValue;
end;

// --- INTERFACE TO ADMAP 3 FILES ----------------------------------------------

CONST
  FeedHole         = $00;
  RightOnlyCode    = $01;
  LeftOnlyCode     = $02;
  UpOnlyCode       = $04;
  RightUpCode      = $05;
  LeftUpCode       = $06;
  DecreaseDiameter = $07;
  DownOnlyCode     = $08;
  RightDownCode    = $09;
  LeftDownCode     = $0A;
  IncreaseDiameter = $0E;
  DrillHoleCode    = $0F;
  HomeCode         = $10;
  LightOnCode      = $11;
  SmallStepCode    = $13;   {0.25 (0.254) mm}
  LightOffCode     = $18;
  LargeStepCode    = $1C;   {2.5  (2.54)  mm}

TYPE
  AdmapStepType = (Undefined, Small, Medium, Large);
  AdmapStatusType = record
                      X, Y: integer;
                      ApertureCode: byte;
                      Step: AdmapStepType;
                      Light: (Off, On)
                    end;

CONST
  InitialAdmapStatus: AdmapStatusType =
    (X: 0;   Y: 0;   ApertureCode: 0;   Step: Undefined;   Light: Off);

  StepSize: array [AdmapStepType] of byte = (0, 1, 5, 10);
  StepAvailable: array [AdmapStepType] of Boolean = (false, true, false, true);
  StepCommandCode: array [AdmapStepType] of byte = (0, SmallStepCode, 0, LargeStepCode);

VAR CurrentAdmapStatus: AdmapStatusType;

function Min (x1, x2: DriverUnits_T): DriverUnits_T;
begin
  if x1 < x2 then Min := x1 else Min := x2
end;

function Max (x1, x2: DriverUnits_T): DriverUnits_T;
begin
  if x1 > x2 then Max := x1 else Max := x2
end;

function Sign (x: DriverUnits_T): DriverUnits_T;
begin
  if x < 0 then Sign := -1 else if x = 0 then Sign := 0 else Sign := 1
end;

// --- ADMAP 3 LOW-LEVEL PROCEDURES --------------------------------------------

procedure Output (CommandCode, NumberOfRepetitions: byte);
  var i: integer;
begin
  for i := 1 to NumberOfRepetitions do
    if not Write (char (CommandCode), 1) then Exit;         (* ??? *)
end;

procedure SwitchLightOn;
begin
  with CurrentAdmapStatus do
    if Light = Off then begin
      Output (LightOnCode, Params [PadExpositions].Value);  (* ??? *)
      Light := On;
    end;
end;

procedure SwitchLightOff;
begin
  with CurrentAdmapStatus do
    if Light = On then begin
      Output (LightOffCode, 1);
      Light := Off;
    end;
end;

procedure SetStepSize (Step: AdmapStepType);
begin
  if CurrentAdmapStatus.Step <> Step then begin
    Output (StepCommandCode [Step], 1);
    CurrentAdmapStatus.Step := Step;
  end;
end;

procedure PerformSteps (StepCommandCode: byte; Length: integer);
  var
    NumbersOfSteps: array [AdmapStepType] of byte;
    PreviousStep, Step: AdmapStepType;
begin
  for Step := Large downto Undefined do
    if StepAvailable [Step] then begin
      NumbersOfSteps [Step] := Length div StepSize [Step];
      Length := Length mod StepSize [Step]
    end
    else
      NumbersOfSteps [Step] := 0;
  PreviousStep := CurrentAdmapStatus.Step;
  for Step := PreviousStep to Large do
    if NumbersOfSteps [Step] > 0 then begin
      SetStepSize (Step);
      Output (StepCommandCode, NumbersOfSteps [Step])
    end;
{$IFOPT R+} {$DEFINE R} {$ENDIF} {$R-}
  for Step := Small to Pred (PreviousStep) do
{$IFDEF R} {$R+} {$ENDIF}
    if NumbersOfSteps [Step] > 0 then begin
      SetStepSize (Step);
      Output (StepCommandCode, NumbersOfSteps [Step]);
    end;
end;

procedure Move (DesiredX, DesiredY: integer);
  const DirectionCommands: array [-1..1, -1..1] of byte =
    ((LeftDownCode,  LeftOnlyCode,  LeftUpCode),
     (DownOnlyCode,  0,             UpOnlyCode),
     (RightDownCode, RightOnlyCode, RightUpCode));
  var DeltaX, DeltaY, HorizontalPart, DiagonalPart, VerticalPart: integer;
begin
  with CurrentAdmapStatus do
    if (X <> DesiredX) or (Y <> DesiredY) then begin
      DeltaX := DesiredX - X;
      DeltaY := DesiredY - Y;
      DiagonalPart   := Min (Abs (DeltaX), Abs (DeltaY));
      HorizontalPart := DeltaX - DiagonalPart * Sign (DeltaX);
      VerticalPart   := DeltaY - DiagonalPart * Sign (DeltaY);

      PerformSteps (DirectionCommands [Sign (DeltaX), 0], Abs (HorizontalPart));
      PerformSteps (DirectionCommands [Sign (DeltaX), Sign (DeltaY)], DiagonalPart);
      PerformSteps (DirectionCommands [0, Sign (DeltaY)], Abs (VerticalPart));

      X := DesiredX;   Y := DesiredY;
    end;
end;

procedure SelectAperture (DesiredCode: byte);
begin
  with CurrentAdmapStatus do
    if ApertureCode <> DesiredCode then begin
      SwitchLightOff;
      PerformSteps (IncreaseDiameter, WorkToolTable [1].Code);
      PerformSteps (DecreaseDiameter, DesiredCode);
      ApertureCode := DesiredCode;
    end;
end;

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
begin
  Write := WriteProc;
  CurrentAdmapStatus := InitialAdmapStatus
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

PROCEDURE OpenLayer (Layer: integer; Color: integer = 0);
begin
end;

PROCEDURE FlashPad (X, Y: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  if (CurrentAdmapStatus.X <> X) or (CurrentAdmapStatus.Y <> Y)
    or (CurrentAdmapStatus.ApertureCode <> WorkToolTable [ToolNumber].Code)
  then begin
    SwitchLightOff;
    Move (X, Y);
    SelectAperture (WorkToolTable [ToolNumber].Code)
  end;
  SwitchLightOn;
end;

PROCEDURE DrawLine (X0, Y0, X1, Y1: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  FlashPad (X0, Y0, ToolNumber);
  Move (X1, Y1);
end;

PROCEDURE CloseEquip;
begin
  SwitchLightOff;
  Move (0, 0);
  Output (HomeCode, 255);
end;

EXPORTS
  GetDescrPtr440,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OpenLayer,
  DrawLine,
  FlashPad,
  CloseEquip;

BEGIN
END.
