(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  parametry procedury OutputBitmap zmnny na typ integer (P93)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY Matrix;

USES DrvTypes, DLL_Util;

TYPE
  ParamIndex_T = (
    Resolution,
    LeftMargin,
    UpperMargin,
    EjectPage);

  Resolution_T = (Res60by60,  Res80by60,   Res90by60,   Res120by60,   Res240by60,
                  Res60by180, Res90by180,  Res120by180, Res180by180,  Res360by180,
                  Res90by216, Res120by216, Res144by216, Res240by216,
                  Res60by60K,                           Res120By60L,  Res240by60Z,
                                                        Res120by216L, Res240by216Z);

CONST
  ToolCount = 0;

  Granularity = 24;

  DriverDescr: DriverDescr_T = (
    Name:         'Matrix Printer';
    MajorVersion: 1;
    MinorVersion: 0;
    Capabilities: [BitMapped, UserTools, ConfigTools];
    NoOfParams:   Ord (High (ParamIndex_T)) + 1;
    FilesName:    'Matrix';
    TargetExt:    'PRN';
    NoOfTools:    ToolCount;
    InitialTools: nil;
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           -2;
    Y0:           -2;
    X1:           195;
    Y1:           280;
    XDividend:    1800;
    YDividend:    1800;
    Divisor:      254;
    Granularity:  Granularity);

  Params: array [ParamIndex_T] of ParamDescr_T = (
   (DescrStr:  '>Resolution (dpi)';
    Value:     Ord (Res180by180);
    ParamType: _Explicite;
    Values:    ' 60 by  60  Esc *| 80 by  60| 90 by  60|120 by  60|240 by  60|'
             + ' 60 by 180| 90 by 180|>120 by 180|1>80 by 180|>360 by 180|'
             + ' 90 by 216|120 by 216|144 by 216|>240 by 216|'
             + ' 60 by  72  Esc K|120 by  72  Esc L|240 by  72  Esc Z|' 
             + '120 by 216  Esc L|240 by 216  Esc Z'),
   (DescrStr:  '>Left Margin (mm)';
    Value:     0;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    200),
   (DescrStr:  '>Upper Margin (mm)';
    Value:     0;
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    200),
   (DescrStr:  '>Eject Page';
    Value:     Ord (false);
    ParamType: _Explicite;
    Values:    'Off|On'));

// -----------------------------------------------------------------------------

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := @ Params [ParamIndex_T (ParamNumber)];
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
  Params [ParamIndex_T (ParamNumber)].Value := ParamValue;
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

// -----------------------------------------------------------------------------

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
  var M, I: integer;
begin
  Write := WriteProc;
  if Params [UpperMargin].Value > 0 then begin
    M := 180 * 10 * longint (Params [UpperMargin].Value) div 254;
    for I := 1 to M div 250 do
      WriteStr (#27'J'#250);  {perform 250/180" line feed}
    WriteStr (#27'J');
    WriteByte (M mod 250);
  end;
end;

PROCEDURE OutputBitmap (BitmapWidth, BitmapHeight, NumOfPlanes: integer; const Planes: Planes_T);
  var Row, Column, Bit, R, I: integer;
      B, M: byte;

  procedure StartBitImageGraphics;
    const ResCodes: array [Resolution_T] of byte = (0,  4,  6,  1,  3,
                                                   32, 38, 33, 39, 40,
                                                    6,  1,  7,  3,
                                              Ord ('K'), Ord ('L'), Ord ('Z'),
                                              Ord ('L'), Ord ('Z'));
  begin
    WriteStr (#13);
    if Params [LeftMargin].Value > 0 then begin
      WriteStr (#27'$');  {set absolute dot position}
      WriteWord (60 * 10 * longint (Params [LeftMargin].Value) div 254);
    end;
    WriteStr (#27);
    if Resolution_T (Params [Resolution].Value) <= Res240by216 then
      WriteStr ('*');
    WriteByte (ResCodes [Resolution_T (Params [Resolution].Value)]);
    WriteWord (BitmapWidth * 8);
  end;

begin
  Row := 0;
  case Resolution_T (Params [Resolution].Value) of
    Res60by60 ..
    Res240by60,
    Res60by60K ..
    Res240by60Z:  while Row <= BitmapHeight - Granularity div 3 do begin
                    StartBitImageGraphics;
                    for Column := 0 to BitmapWidth - 1 do begin
                      for Bit := 7 downto 0 do begin
                        M := 1 shl Bit;
                        B := 0;
                        for R := 0 to 7 do
                          if (Planes [0]^ [Row + R]^ [Column] and M) <> 0 then
                            B := B or ($80 shr R);
                        WriteByte (B);
                      end;
                    end;
                    WriteStr (#27'J'#24);  {8/60" line feed}
                    Inc (Row, Granularity div 3);
                  end;
    Res60by180 ..
    Res360by180:  while Row <= BitmapHeight - Granularity do begin
                    StartBitImageGraphics;
                    for Column := 0 to BitmapWidth - 1 do begin
                      for Bit := 7 downto 0 do begin
                        M := 1 shl Bit;

                        B := 0;
                        for R := 0 to 7 do
                          if (Planes [0]^ [Row +  0 + R]^ [Column] and M) <> 0 then
                            B := B or ($80 shr R);
                        WriteByte (B);

                        B := 0;
                        for R := 0 to 7 do
                          if (Planes [0]^ [Row +  8 + R]^ [Column] and M) <> 0 then
                            B := B or ($80 shr R);
                        WriteByte (B);

                        B := 0;
                        for R := 0 to 7 do
                          if (Planes [0]^ [Row + 16 + R]^ [Column] and M) <> 0 then
                            B := B or ($80 shr R);
                        WriteByte (B);
                      end;
                    end;
                    WriteStr (#27'J'#24);  {8/60" line feed}
                    Inc (Row, Granularity);
                  end;
    Res120by216,
    Res144by216,
    Res240by216,
    Res120by216L,
    Res240by216Z: while Row <= BitmapHeight - Granularity do begin
                    for I := 0 to 2 do begin
                      StartBitImageGraphics;
                      for Column := 0 to BitmapWidth - 1 do begin
                        for Bit := 7 downto 0 do begin
                          M := 1 shl Bit;
                          B := 0;
                          for R := 0 to 7 do
                            if (Planes [0]^ [Row + 3 * R + I]^ [Column] and M) <> 0 then
                              B := B or ($80 shr R);
                          WriteByte (B);
                        end;
                      end;
                      if i < 2 then
                        WriteStr (#27'J'#1)  {1/216" line feed}
                      else
                        WriteStr (#27'J'#22);  {1/216" + 7/72" line feed}
                    end;
                    Inc (Row, Granularity);
                  end;
  end;
end;

PROCEDURE CloseEquip;
begin
  WriteStr (#13);         {carriage return}
  if Boolean (Params [EjectPage].Value) then
    WriteStr (^L);        {form feed}
end;

EXPORTS
  GetDescrPtr440,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OutputBitmap,
  CloseEquip;

BEGIN
END.
