(* -----------------------------------------------------------------------------

   =====================================

    FORMICA 4.40 LAYOUT
    Copyright (C) 1986-2004 Petr Horsk

   =====================================


--- ZMNY OPROTI REL. R90: -----------------------------------------------------
1)  funkce GetDescrPtr pejmenovna pro rozlien verz na GetDescrPtr440 (P93)
2)  do funkce OpenLayer doplnn parametr Color (P93)
----------------------------------------------------------------------------- *)

{$LONGSTRINGS OFF}

LIBRARY Posalux;

USES DrvTypes, DLL_Util;

TYPE
  ParamIndex_T = (
    NewLineCode,
    EvenParityEnabled,
    LeadingFeedHoles,
    TrailingFeedHoles);

  NewLineCode_T = (Lf, CrLf);

CONST
  ToolCount = 9;
  ToolTable: array [1..ToolCount] of ToolDescr_T = (
    (DescrStr: 'T1 (0.9 mm)'; Shape: CircleTool; Width:  900; Height: 0; Usage: [UsageDrill]; Code:  1),
    (DescrStr: 'T2 (1.1 mm)'; Shape: CircleTool; Width: 1100; Height: 0; Usage: [UsageDrill]; Code:  2),
    (DescrStr: 'T3 (1.6 mm)'; Shape: CircleTool; Width: 1600; Height: 0; Usage: [UsageDrill]; Code:  3),
    (DescrStr: 'T4 (2.0 mm)'; Shape: CircleTool; Width: 2000; Height: 0; Usage: [UsageDrill]; Code:  4),
    (DescrStr: 'T5 (2.5 mm)'; Shape: CircleTool; Width: 2500; Height: 0; Usage: [UsageDrill]; Code:  5),
    (DescrStr: 'T6 (3.2 mm)'; Shape: CircleTool; Width: 3200; Height: 0; Usage: [UsageDrill]; Code:  6),
    (DescrStr: 'T7 (4.0 mm)'; Shape: CircleTool; Width: 4000; Height: 0; Usage: [UsageDrill]; Code:  7),
    (DescrStr: 'T8 (5.0 mm)'; Shape: CircleTool; Width: 5000; Height: 0; Usage: [UsageDrill]; Code:  8),
    (DescrStr: 'T9 (6.0 mm)'; Shape: CircleTool; Width: 6000; Height: 0; Usage: [UsageDrill]; Code:  9));

  DriverDescr: DriverDescr_T = (
    Name:         'Posalux 4.0 PH 1995';
    MajorVersion: 4;
    MinorVersion: 0;
    Capabilities: [Optimizable, DrillHoles];
    NoOfParams:   Ord (High (ParamIndex_T)) + 1;
    FilesName:    'Posalux';
    TargetExt:    '';
    NoOfTools:    ToolCount;
    InitialTools: @ ToolTable [1];
    MaxNoOfTools: MaxToolCount;
    ToolTablePtr: @ WorkToolTable;
    X0:           0;
    Y0:           0;
    X1:           800;
    Y1:           600;
    XDividend:    100;
    YDividend:    100;
    Divisor:      1);

  Params: array [ParamIndex_T] of ParamDescr_T = (
   (DescrStr:  '>New Line Code';
    Value:     Ord (CrLf);
    ParamType: _Explicite;
    Values:    '>LF|>CR-LF'),
   (DescrStr:  '>Even Parity';
    Value:     Ord (false);
    ParamType: _Explicite;
    Values:    'Off|On'),
   (DescrStr:  '>Leading Feed Holes';
    Value:     0;  {200}
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    1000),
   (DescrStr:  '>Trailing Feed Holes';
    Value:     0;  {200}
    ParamType: _Numeric;
    MinVal:    0;
    MaxVal:    1000));

FUNCTION GetDescrPtr440: DescrPtr_T;
begin
  Result := @ DriverDescr;
end;

FUNCTION GetParamPtr (ParamNumber: integer): ParamPtr_T;
begin
  Result := @ Params [ParamIndex_T (ParamNumber)];
end;

PROCEDURE SetParamValue (ParamNumber: integer; ParamValue: integer);
begin
  Params [ParamIndex_T (ParamNumber)].Value := ParamValue;
end;

// -----------------------------------------------------------------------------

procedure Output (AsciiString: string);
  const
    Cr:  char = ^M;
    CrP: char = char (byte (^M) or $80);
  var i: integer;
begin
  for i := 1 to Length (AsciiString) do begin
    if (AsciiString [i] = ^J)
      and (NewLineCode_T (Params [NewLineCode].Value) = CrLf)
    then
      if Boolean (Params [EvenParityEnabled].Value) then
        WriteByte (Ord (CrP))
      else
        WriteByte (Ord (Cr));
    if Boolean (Params [EvenParityEnabled].Value) then
      WriteByte (Ord (EvenParity (AsciiString [i])))
    else
      WriteByte (Ord (AsciiString [i]));
  end;
end;

PROCEDURE OpenEquip (WriteProc: WriteProc_T; X0, Y0, X1, Y1: DriverUnits_T);
  var i: integer;
begin
  Write := WriteProc;
  for i := 1 to Params [LeadingFeedHoles].Value do
    Output (#0);
  Output ('%%'^J);
end;

PROCEDURE Comment (const CommentString: string);
begin
end;

PROCEDURE OpenLayer (Layer: integer; Color: integer = 0);
begin
end;

PROCEDURE DrillHole (X, Y: DriverUnits_T; ToolNumber: ToolNumber_T);
begin
  Output ('X' + IntStr (X) + 'Y' + IntStr (Y) 
        + 'T' + IntStr (WorkToolTable [ToolNumber].Code) + ^J);
end;

PROCEDURE CloseEquip;
  var i: integer;
begin
  Output ('M30'^J);
  for i := 1 to Params [TrailingFeedHoles].Value do
    Output (#0);
end;

EXPORTS
  GetDescrPtr440,
  GetParamPtr,
  SetParamValue,
  OpenEquip,
  Comment,
  OpenLayer,
  DrillHole,
  CloseEquip;

BEGIN
END.
